package data.scripts.campaign.bases;

import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.FleetAssignment;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.impl.campaign.fleets.BaseLimitedFleetManager;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactory.MercType;
import static com.fs.starfarer.api.impl.campaign.fleets.FleetFactory.MercType.BOUNTY_HUNTER;
import static com.fs.starfarer.api.impl.campaign.fleets.FleetFactory.MercType.PATROL;
import static com.fs.starfarer.api.impl.campaign.fleets.FleetFactory.MercType.PRIVATEER;
import static com.fs.starfarer.api.impl.campaign.fleets.FleetFactory.MercType.SCOUT;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV3;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParamsV3;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;
import java.util.ArrayList;
import java.util.List;

import static data.scripts.campaign.bases.VayraRaiderBaseManager.FP_BOUNTY_HUNTER;
import static data.scripts.campaign.bases.VayraRaiderBaseManager.FP_PATROL;
import static data.scripts.campaign.bases.VayraRaiderBaseManager.FP_PRIVATEER;
import static data.scripts.campaign.bases.VayraRaiderBaseManager.FP_SCOUT;
import data.scripts.campaign.bases.VayraRaiderBaseManager.RaiderData;
import static data.scripts.campaign.bases.VayraRaiderBaseManager.SPAWN_WEIGHT_BOUNTY_HUNTER;
import static data.scripts.campaign.bases.VayraRaiderBaseManager.SPAWN_WEIGHT_PATROL;
import static data.scripts.campaign.bases.VayraRaiderBaseManager.SPAWN_WEIGHT_PRIVATEER;
import static data.scripts.campaign.bases.VayraRaiderBaseManager.SPAWN_WEIGHT_SCOUT;
import org.apache.log4j.Logger;

public class VayraRaiderNonEventFleetManager extends BaseLimitedFleetManager {

    public static Logger thisLog = Global.getLogger(VayraRaiderNonEventFleetManager.class);

    public VayraRaiderNonEventFleetManager(RaiderData data) {
        this.raiderData = data;
        if (raiderData == null) {
            thisLog.error("created VayraRaiderNonEventFleetManager with null raiderData");
        }
        if (raiderData.raiderFactionId == null) {
            thisLog.error("created VayraRaiderNonEventFleetManager with null raiderData.raiderFactionId");
        }
        FactionAPI faction = Global.getSector().getFaction(raiderData.raiderFactionId);
        if (faction == null) {
            thisLog.error("created VayraRaiderNonEventFleetManager with null faction");
        }
    }

    protected RaiderData raiderData;

    public static class NonEventRaiderManagedFleetData {

        public float startingFleetPoints = 0;
        public CampaignFleetAPI fleet;

        public NonEventRaiderManagedFleetData(CampaignFleetAPI fleet) {
            this.fleet = fleet;
        }
    }

    protected List<NonEventRaiderManagedFleetData> fleetsActive = new ArrayList<>();

    @Override
    protected int getMaxFleets() {
        int count = 0;
        int limit = 10;
        for (MarketAPI market : Global.getSector().getEconomy().getMarketsCopy()) {
            String fid = market.getFactionId();
            if (fid != null && raiderData != null && raiderData.raiderFactionId != null && fid.equals(raiderData.raiderFactionId)) {
                count++;
                if (market.getSize() > 3) {
                    count++;
                }
            }
        }
        if (count > limit) {
            count = limit;
        }
        return count;
    }

    public CampaignFleetAPI spawnFleet(MarketAPI spawnMarket, MercType type, MarketAPI target) {

        if (raiderData == null || raiderData.raiderFactionId == null) {
            thisLog.warn("raiderData or raiderData.raiderFactionId was null - returning null");
            return null;
        }

        FactionAPI faction = Global.getSector().getFaction(raiderData.raiderFactionId);
        if (spawnMarket == null) {
            WeightedRandomPicker<MarketAPI> picker = new WeightedRandomPicker<>();
            for (MarketAPI market : Misc.getFactionMarkets(faction)) {
                picker.add(market, market.getSize());
            }

            spawnMarket = picker.pick();
            if (spawnMarket == null) {
                thisLog.warn("couldn't pick spawnMarket - returning null");
                return null;
            }
        }

        if (target == null) {
            target = pickTarget(spawnMarket);
            if (target == null) {
                thisLog.warn("couldn't pick target - returning null");
                return null;
            }
        }

        if (type == null) {
            WeightedRandomPicker<MercType> picker = new WeightedRandomPicker<>();
            picker.add(SCOUT, SPAWN_WEIGHT_SCOUT);
            picker.add(BOUNTY_HUNTER, SPAWN_WEIGHT_BOUNTY_HUNTER);
            picker.add(PRIVATEER, SPAWN_WEIGHT_PRIVATEER);
            picker.add(PATROL, SPAWN_WEIGHT_PATROL);

            type = picker.pick();
        }

        float combat = 0f;
        float tanker = 0f;
        float freighter = 0f;
        String fleetType = type.fleetType;
        switch (type) {
            case SCOUT:
                combat = Math.round(FP_SCOUT + (float) Math.random() * FP_SCOUT);
                break;
            case PRIVATEER:
                combat = Math.round(FP_PRIVATEER + (float) Math.random() * FP_PRIVATEER);
                break;
            case BOUNTY_HUNTER:
                combat = Math.round(FP_BOUNTY_HUNTER + (float) Math.random() * FP_BOUNTY_HUNTER);
                break;
            case PATROL:
                combat = Math.round(FP_PATROL + (float) Math.random() * FP_PATROL);
                break;
        }

        FleetParamsV3 params = new FleetParamsV3(
                spawnMarket,
                target.getLocation(), raiderData.raiderFactionId, // quality will always be reduced by non-market-faction penalty, which is what we want 
                null,
                fleetType,
                combat, // combatPts
                freighter, // freighterPts 
                tanker, // tankerPts
                0f, // transportPts
                0f, // linerPts
                0f, // utilityPts
                0f // qualityMod
        );

        CampaignFleetAPI fleet = FleetFactoryV3.createFleet(params);
        thisLog.info(String.format("Spawning fleet via SpawnFleet, spawnMarket: [%s], target: [%s], combat points: [%s]", spawnMarket.getName(), target.getName(), (int) combat));

        if (fleet == null || fleet.isEmpty()) {
            thisLog.info(String.format("Fleet was null. Nevermind"));
            return null;
        }

        thisLog.info(String.format("Spawned fleet via SpawnFleet, Actual Fleet Points: [%s], Fleet Size: [%s]", fleet.getFleetPoints(), fleet.getFleetSizeCount()));

        SectorEntityToken entity = spawnMarket.getPrimaryEntity();
        entity.getContainingLocation().addEntity(fleet);
        fleet.setLocation(entity.getLocation().x, entity.getLocation().y);
        MarketAPI source = Misc.getSourceMarket(fleet);
        if (source == null) {
            thisLog.info(String.format("Source was null. Nevermind"));
            return null;
        }

        Vector2f dest = Misc.getPointAtRadius(target.getLocationInHyperspace(), 1500);
        LocationAPI hyper = Global.getSector().getHyperspace();
        SectorEntityToken token = hyper.createToken(dest.x, dest.y);

        fleet.addAssignment(FleetAssignment.GO_TO_LOCATION, token, 1000,
                "travelling to " + target.getName());
        fleet.addAssignment(FleetAssignment.ATTACK_LOCATION, target.getPrimaryEntity(), 20,
                raiderData.raidingActionText + " " + target.getName());
        fleet.addAssignment(FleetAssignment.GO_TO_LOCATION, source.getPrimaryEntity(), 1000,
                "returning to " + source.getName());
        fleet.addAssignment(FleetAssignment.ORBIT_PASSIVE, source.getPrimaryEntity(), 2f + 2f * (float) Math.random(),
                raiderData.raidingStanddownText);
        fleet.addAssignment(FleetAssignment.GO_TO_LOCATION_AND_DESPAWN, source.getPrimaryEntity(), 1000);

        return fleet;
    }

    protected MarketAPI pickTarget(MarketAPI market) {

        WeightedRandomPicker<MarketAPI> picker = new WeightedRandomPicker<>();
        WeightedRandomPicker<MarketAPI> closePicker = new WeightedRandomPicker<>();
        FactionAPI faction = Global.getSector().getFaction(raiderData.raiderFactionId);

        if (market == null) {
            return null;
        }

        for (MarketAPI targetMarket : Global.getSector().getEconomy().getMarketsCopy()) {

            if (targetMarket == null) {
                continue;
            }

            String targetFactionId = targetMarket.getFactionId();
            float repWeight = -faction.getRelationship(targetFactionId);
            repWeight = Math.max(0f, repWeight);
            StarSystemAPI system = market.getStarSystem();
            StarSystemAPI targetSystem = targetMarket.getStarSystem();

            if (repWeight <= 0 || targetSystem == null) {
                continue;
            }

            if (raiderData.raidTargetWeights.containsKey(targetFactionId)) {
                picker.add(targetMarket, targetMarket.getSize() * repWeight * raiderData.raidTargetWeights.get(targetFactionId));
                if (system.equals(targetSystem)) {
                    closePicker.add(targetMarket, targetMarket.getSize() * repWeight * raiderData.raidTargetWeights.get(targetFactionId));
                }
            } else {
                picker.add(targetMarket, targetMarket.getSize() * repWeight);
                if (system.equals(targetSystem)) {
                    closePicker.add(targetMarket, targetMarket.getSize() * repWeight);
                }
            }
        }

        MarketAPI newTarget = picker.pick();

        if (!closePicker.isEmpty()) {
            newTarget = closePicker.pick();
        }

        return newTarget;
    }

    @Override
    public void advance(float amount) {

        float days = Global.getSector().getClock().convertToDays(amount);

        tracker.advance(days * getSpawnRateMult());
        if (!tracker.intervalElapsed()) {
            return;
        }

        float next = getNextInterval();
        tracker.setInterval(next, next);

        Global.getSettings().profilerBegin(this.getClass().getSimpleName() + ".advance()");
        List<NonEventRaiderManagedFleetData> remove = new ArrayList<>();
        for (NonEventRaiderManagedFleetData fleetData : fleetsActive) {
            if (fleetData.fleet.getContainingLocation() == null || !fleetData.fleet.getContainingLocation().getFleets().contains(fleetData.fleet)) {
                remove.add(fleetData);
                thisLog.info("Cleaning up orphaned fleet [" + fleetData.fleet.getNameWithFaction() + "] of [" + fleetData.fleet.getFleetSizeCount() + "] ships");
            }
        }
        fleetsActive.removeAll(remove);

        int max = getMaxFleets();

        if (raiderData != null && raiderData.raiderFactionId != null) {
            if (fleetsActive.size() < max) {
                thisLog.info(fleetsActive.size() + " out of a maximum " + max + " nonevent fleets in play for [" + raiderData.raiderFactionId + "]");
                CampaignFleetAPI fleet = spawnFleet();
                if (fleet != null) {
                    fleet.addEventListener(this);
                    NonEventRaiderManagedFleetData fleetData = new NonEventRaiderManagedFleetData(fleet);
                    fleetsActive.add(fleetData);
                    thisLog.info("Spawned nonevent raider fleet [" + fleet.getNameWithFaction() + "] at hyperloc " + fleet.getLocationInHyperspace());
                } else {
                    thisLog.info("Could not spawn fleet - returned null");
                }
            } else {
                thisLog.info("Maximum number of " + max + " nonevent fleets already in play for [" + raiderData.raiderFactionId + "]");
            }
        } else {
            thisLog.error("raiderData or raiderData.raiderFactionId was null!");
        }
        Global.getSettings().profilerEnd();
    }

    @Override
    protected CampaignFleetAPI spawnFleet() {
        return spawnFleet(null, null, null);
    }
}
